

# Calibration and generalized calibration

# Package sampling
# starting from package sampling vignettes

library(sampling)

############
## Example 1
############
# matrix of sample calibration variables 
Xs=cbind(
  c(1,1,1,1,1,0,0,0,0,0),
  c(0,0,0,0,0,1,1,1,1,1),
  c(1,2,3,4,5,6,7,8,9,10)
)
# inclusion probabilities
piks=rep(0.2,times=10)
# vector of population totals
total=c(24,26,290)
# the g-weights using the truncated method
g=calib(Xs,d=1/piks,total,method="truncated",bounds=c(0.75,1.2))
# the calibration estimator of X is equal to 'total' vector
tcal=t(g/piks)%*%Xs
# the g-weights are between lower and upper bounds
g


############
## Example 2
############
# Example of g-weights (linear, raking, truncated, logit),
# with the data of Belgian municipalities as population.
# Firstly, a sample is selected by means of Poisson sampling.
# Secondly, the g-weights are calculated.
data(belgianmunicipalities)
attach(belgianmunicipalities)
# matrix of calibration variables for the population
X=cbind(
  Men03/mean(Men03),
  Women03/mean(Women03),
  Diffmen,
  Diffwom,
  TaxableIncome/mean(TaxableIncome),
  Totaltaxation/mean(Totaltaxation),
  averageincome/mean(averageincome),
  medianincome/mean(medianincome))
# selection of a sample with expectation size equal to 200
# by means of Poisson sampling
# the inclusion probabilities are proportional to the average income 
pik=inclusionprobabilities(averageincome,200)
N=length(pik)               # population size
s=UPpoisson(pik)            # sample
Xs=X[s==1,]                 # sample matrix of calibration variables
piks=pik[s==1]              # sample inclusion probabilities
n=length(piks)              # sample size
# vector of population totals of the calibration variables
total=c(t(rep(1,times=N))%*%X)  
# the population total 
total
# computation of the g-weights
# by means of different calibration methods.
g1=calib(Xs,d=1/piks,total,method="linear")
g2=calib(Xs,d=1/piks,total,method="raking")
g3=calib(Xs,d=1/piks,total,method="truncated",bounds=c(0.5,1.5))
g4=calib(Xs,d=1/piks,total,method="logit",bounds=c(0.5,1.5))
# In some cases, the calibration does not exist
# particularly when bounds are used.
# if the calibration is possible, the calibration estimator of Xs is printed
if(checkcalibration(Xs,d=1/piks,total,g1)$result) 
  print(c((g1/piks) %*% Xs)) else print("error")
if(!is.null(g2))
  if(checkcalibration(Xs,d=1/piks,total,g2)$result) 
    print(c((g2/piks) %*% Xs)) else print("error")
if(!is.null(g3))
  if(checkcalibration(Xs,d=1/piks,total,g3)$result) 
    print(c((g3/piks) %*% Xs)) else print("error")
if(!is.null(g4))
  if(checkcalibration(Xs,d=1/piks,total,g4)$result) 
    print(c((g4/piks) %*% Xs)) else print("error")

################################
# Example 3
################################
# This is an example of using the calib function for calibration and 
# nonresponse adjustment (with response homogeneity groups).
# We create the population data frame (the population size is N = 250)

# there are four variables: state, region, income and sex;
# the state variable has 2 categories: 'A' and 'B'; 
# the region variable has 3 categories: 1, 2, 3 (regions within states);
# the income and sex variables are randomly generated using the uniform distribution.

data = rbind(matrix(rep("A", 150), 150, 1, byrow = TRUE),
             matrix(rep("B", 100), 100, 1, byrow = TRUE))
data = cbind.data.frame(data, c(rep(1, 60), rep(2,50), rep(3, 60), rep(1, 40), rep(2, 40)),
                          1000 * runif(250))
sex = runif(nrow(data))
for (i in 1:length(sex)) if (sex[i] < 0.3) sex[i] = 1 else sex[i] = 2
data = cbind.data.frame(data, sex)
names(data) = c("state", "region", "income", "sex")
summary(data)

# We compute the population stratum sizes:

table(data$state)

# We select a stratified sample. The state variable is used as a stratification variable. 
# The sample stratum sizes are 25 and 20, respectively. The method is 'srswor' 
# (equal probability, without replacement).

s=strata(data,c("state"),size=c(25,20), method="srswor")

# we obtained the observed data

s=getdata(data,s)

# The status variable is used in the rhg_strata function. 
# The status column is added to s (1 - sample respondent, 0 otherwise); 
# it is randomly generated using the uniform distribution.
# The response probability for all units is 0.3.
# here we are considering missing answer problems

status=runif(nrow(s))
for(i in 1:length(status))
if(status[i]<0.3) status[i]=0 else status[i]=1
s=cbind.data.frame(s,status)


# We compute the response homeogeneity groups using the region variable:

s=rhg_strata(s,selection="region")

# selected only the sample respondents

sr=s[s$status==1,]

# We create the population data frame of sex and region indicators:

X=cbind(disjunctive(data$sex),disjunctive(data$region))
?disjunctive

# transforms a categorical variable into a matrix of indicators. 
# The values of the categorical 
# variable are integer numbers (positive or negative).

# We compute the population totals for each sex and region:

total=c(t(rep(1,nrow(data)))%*%X)

# The first method consists in calibrating with all strata. 
# The respondent data frame of sex and region indicators is created. 
# The initial weights using the inclusion prob. and the response probabilities are computed.

Xs = X[sr$ID_unit,]
d = 1/(sr$Prob * sr$prob_resp)
summary(d)

# We compute the g-weights using the linear method:

g = calib(Xs, d, total, method = "linear")
summary(g)

# The final weights are:

w=d*g
summary(w)


# We check the calibration:

checkcalibration(Xs, d, total, g)

# The second method consists in calibrating in each stratum.
# The respondent data frame of sex and region indicators is created in each stratum. 
# The initial weights using the inclusion prob. and response probabilities are 
# computed in each stratum.

cat("stratum 1\n")

data1=data[data$state=='A',]
X1=X[data$state=='A',]
total1=c(t(rep(1, nrow(data1))) %*% X1)
sr1=sr[sr$Stratum==1,]
Xs1=X[sr1$ID_unit,]
d1 = 1/(sr1$Prob * sr1$prob_resp)
g1=calib(Xs1, d1, total1, method = "linear")
checkcalibration(Xs1, d1, total1, g1)

cat("stratum 2\n")

data2=data[data$state=='B',]
X2=X[data$state=='B',]
total2=c(t(rep(1, nrow(data2))) %*% X2)
sr2=sr[sr$Stratum==2,]
Xs2=X[sr2$ID_unit,]
d2 = 1/(sr2$Prob * sr2$prob_resp)
g2=calib(Xs2, d2, total2, method = "linear")
checkcalibration(Xs2, d2, total2, g2)



library(survey)
data(api)
?api
# Student performance in California schools
# The full population data in apipop are a data frame with 6194 
# observations on the following 37 variables.
dclus1<-svydesign(id=~dnum, weights=~pw, data=apiclus1, fpc=~fpc)
?api
# dnum: District number
# apiclus1 is a cluster sample of school districts
# > dim(apiclus1)
# [1] 183  39
cal_names(~stype, dclus1)
# stype: elementary/middle/high school
# The formula argument specifies a model matrix, 
# and the population argument is the population column sums of this matrix. 
# The function cal_names shows what the column names of this model matrix will be.

pop.totals<-c(`(Intercept)`=6194, stypeH=755, stypeM=1018)

## For a single factor variable this is equivalent to
## postStratify

(dclus1g<-calibrate(dclus1, ~stype, pop.totals))

svymean(~api00, dclus1g)
svytotal(~enroll, dclus1g)
svytotal(~stype, dclus1g)

## Make weights constant within school district
(dclus1agg<-calibrate(dclus1, ~stype, pop.totals, aggregate=1))
# aggregate.stage	An integer. 
# If not NULL, make calibration weights constant within sampling units at this stage.
svymean(~api00, dclus1agg)
svytotal(~enroll, dclus1agg)
svytotal(~stype, dclus1agg)

## Now add sch.wide
(dclus1g2 <- calibrate(dclus1, ~stype+sch.wide, c(pop.totals, sch.wideYes=5122)))

svymean(~api00, dclus1g2)
svytotal(~enroll, dclus1g2)
svytotal(~stype, dclus1g2)

## Finally, calibrate on 1999 API and school type

(dclus1g3 <- calibrate(dclus1, ~stype+api99, c(pop.totals, api99=3914069)))

svymean(~api00, dclus1g3)
svytotal(~enroll, dclus1g3)
svytotal(~stype, dclus1g3)
