

#install.packages(c('sampling','survey'))


library(sampling)
# Methods for sampling: we can find in this packages: (a) function for drawing sampling and calibration
# (b) HT and calibration estimators  (c) implement sampling schemes
# library(survey)
# Methods for the analysis of  complex survey data

# These packages encompass function for dealing with the most common sampling design:
# - computation of inclusion probability
# - sample extraction
# - estimation of population quantities

# Some interesting datasets to exercise with are also provide
# Let's begin by understanding how the survey and sampling package works. 
# Go to packages window in RStudio, click on survey or sampling
# You can find some usefull example in "Vignettes"

# Packages sampling implements the selection of many equal and unequal sampling designs 
# and most of them are without replacement.
# The outputs of functions dedicated to sample designs are N-dimensional vectors containing zeros for 
# non selected-individuals and positive values for the selected individuals.
# Introduction to some sampling schemes:
# a) Simple random sampling (SRS) provides a natural starting point for a discussion of probability sampling
# methods. It is the simplest method and it underlies many of the more complex methods.
# Simple random sampling is a sampling scheme with the property that any of the
# possible subsets of n distinct elements, from the population of N elements, is equally likely to be the
# chosen sample.
# Every element in the population has the same probability of being selected for the sample, and the joint
# probabilities of sets of elements being selected are equal

# b) Poisson sampling assumes that the random mechanism to generate the data can be described
# by a Poisson distribution. 
# It is useful for modeling counts or events that occur randomly over a fixed period of time 
# or in a fixed space. Often it is useful when the probability of any particular incidence 
# happening is very small while the number of incidences is very large. 
# (This is very much like a binomial distribution where success probability 
# of a trial is very very small but the number of trials n is very very large.
# This is known as the limiting condition).

# c) Systematic sampling is a type of probability sampling method in which sample members from a 
# larger population are selected according to a random starting point and a fixed, periodic interval.
# This interval, called the sampling interval, is calculated by dividing the population size by the 
# desired sample size. Despite the sample population being selected in advance, 
# systematic sampling is still thought of as being random if the periodic interval is determined 
# beforehand and the starting point is random.

# d) Stratified sampling is a probability sampling technique wherein the researcher divides the entire 
# population into different subgroups or strata, then randomly selects the final subjects proportionally 
# from the different strata.




# Exercise using packages Sampling (Starting from Till 2010)

# Load the data set
data(belgianmunicipalities)
?belgianmunicipalities
belgianmunicipalities=belgianmunicipalities[order(belgianmunicipalities$Province),]
dim(belgianmunicipalities)
head(belgianmunicipalities)
tail(belgianmunicipalities)
# 
summary(belgianmunicipalities)
hist(belgianmunicipalities$TaxableIncome)

# For the stratified sampling, strata are the Provinces
table(belgianmunicipalities$Province)
# Computes the inclusion probabilities using the 'Tot04' variable and a sample size of 200
# Computes the first-order inclusion probabilities from a vector of positive numbers (for a probability
# proportional-to-size sampling design).
pik = inclusionprobabilities(belgianmunicipalities$Tot04,200)
# Defines the population size
N = length(pik)
# Defines the sample size
n = sum(pik)
Nh = table(belgianmunicipalities$Province)
Nb = sum(Nh)
nh = round(n*(Nh/N),0)  # Proportional allocation
sum(nh)
# Defines the number of simulations (for an accurate result, increase this value)
sim = 1000
ss = array(0, c(sim, 4))
# Defines the variable of interest ('Taxable Income')
y = belgianmunicipalities$TaxableIncome
# Simulation and computation of the Horvitz-Thompson estimator
ht = numeric(4)
# 4 sampling designs: 2 with equal probabilities and 2 with unequal probabilities
for (i in 1:sim) {
  cat("Step ", i, "\n")
  set.seed(i)
  s = UPpoisson(pik)
  # UPpoisson gives Poisson sampling with unequal inclusion probabilities contained in \pi_{k}
  # for constant \pi_{k}=\pi for all k in U, we obtain the Bernoulli sampling
  ht[1] = HTestimator(y[s == 1], pik[s == 1])
  s = UPsystematic(pik)
  # UPsystematic implements the sistematic sampling with unequal inclusion probabilities contained in the
  # N- dimensional vector pik. 
  # gives systematic sampling with inclusion probabilities \pi_{k}=nx_{k}/sum_{U}x_{k} where
  # x_{k} is an auxiliary variable.
  # for \pi_{k}=n/N we have the usual systematic sampling
  ht[2] = HTestimator(y[s == 1], pik[s == 1])
  s = srswor(n, N)
  # srswor implements the selection of a simple random sample without replacement of size n from a population
  # of size N. The output contains a vectore of size N containing n ones and N-n zeros
  ht[3] = HTestimator(y[s == 1], rep(n/N, n))
  st = strata(belgianmunicipalities,stratanames=c("Province"),size=c(24, 38, 22, 22, 23, 29, 15, 15, 13),
              method="srswor")
  # a stratified sampling is selected using the function strata
  x=getdata(belgianmunicipalities, st)
  ht[4] = HTstrata(x$TaxableIncome,x$Prob,x$Stratum,description=TRUE)
  ss[i, ] = ss[i, ] + ht
}
# boxplots of the estimators
colnames(ss)=c("poisson", "syst","srswor","strata")
boxplot(data.frame(ss), las = 3, title="Accuracy if the HT estimator under different sampling designs")
# Accuracy: Ability of an estimator to be correct on average
sum(as.numeric(belgianmunicipalities$TaxableIncome))
apply(ss, 2, mean)

rm(list=ls())

# Now we want to compute the variances for the HT estimator of the taxable income
# under the poisson sampling scheme 
# varHT(y,pikl,method) Computes variance estimators of the Horvitz-Thompson estimator 
# of the population total:
# y vector of the variable of interest; its length is equal to n, the sample size.
# pikl matrix of second-order inclusion probabilities; its dimension is nxn.
# method if 1, an unbiased variance estimator is computed (the HT variance estimator); 
# if 2, the Sen-Yates-Grundy variance estimator for fixed sample size is computed; 
# by default, the method is 1. 


library(sampling)
data(belgianmunicipalities)
pik = inclusionprobabilities(belgianmunicipalities$Tot04,200)
sim = 1000
ss = array(0, c(sim, 2))
# Defines the variable of interest ('Taxable Income')
y = belgianmunicipalities$TaxableIncome
# Simulation and computation of the Horvitz-Thompson estimator
ht = NULL
vht = NULL

for (i in 1:sim) {
  cat("Step ", i, "\n")
  set.seed(i)
  s = UPpoisson(pik)
  # gives Poisson sampling with inclusion probabilities \pi_{k}
  # for constant \pi_{k}=\pi for all k in U, we obtain the Bernoulli sampling
  ht[i] = HTestimator(y[s == 1], pik[s == 1])
  pikl=outer(pik,pik,"*")
  diag(pikl)=pik
  # Computes the variance estimator (method=1, the sample size is not fixed)
  vht[i]=varHT(y[s==1],pikl[s==1,s==1],1)
}

head(ht)
head(vht)
