

# Stratified Sampling

 # The division of the population into groups represents an example of a 
# priori information on the population that it is reasonable to try to exploit in the construction of the sampling plan.

# Frequently, before undertaking an investigation, some information that can be used in the sampling strategy to improve the
#accuracy of the estimators is available for all units of the population. If, for example, we need to form a sample of municipalities, 
#the data relative to their geographical location, 
#the demographic dimension, the prevailing economic activity, etc. are normally known before the extraction.

# Stratification is a methodology that allows you to use this type of information (called additional or auxiliary information) 
# to improve the sampling design. It consists in the classification of the population in sub-populations, called strata, 
# on the basis of auxiliary information and in the selection of samples independent of each layer. 
# 
# The major advantages of stratification stem from the fact that the sample size in the layers rather than being determined
# by the randomness of the extraction - as would be the case in simple random sampling - is under the control of the sampler.

# The main advantages of the stratified sample are:

# I) it is possible to obtain more efficient estimators (especially if strata are homogeneous);

# II) the strata are sampled independently of each other. This
# allows, for example, to adopt different methods of detection in different layers;

# III) the strata form subpopulations for which we extract simple random samples. 
# We can estimate parameters for these subpopulations by applying simple random sampling theory.

# Since simple random sampling was assumed in the strata, the results seen remain valid for each strata taken separately. 
# Therefore, the sample strata averages are correct estimators of the respective strata averages and the variances are obtained 
# immediately from the expression of the variance of the simple random sampling. The new problems only concern:
# 1. how to combine stratum estimates to obtain an estimator of the general averge (or total);
# 2. how to estimate the estimator variance of the mean (total).

# Allocating a stratified sample means deciding how the overall sample size n should be divided into the L strata. 
# Samples are often formed by applying the same sampling fraction in all the layers. 
# In this case they are proportional to the layer of origin and the stratification itself is called proportional. 
# With this type of stratification, as we will see, we normally have the guarantee of obtaining better (more precise)
# estimators than those that would come from a simple random sample.
# Other forms of stratification do not guarantee this objective but allow the pursuit of others that may be of considerable 
# importance in the survey.

#
#To increase the efficiency of the estimators the idea is to more intensively sample the layers that are characterized by 
#a higher variability. 
# In this spirit an optimal solution was derived: Neyman's allocation.
# In order to be able to use the packages sampling and survey 
# in R it is necessary first of all to make them active in the memory of R through the function


library(sampling)

set.seed(1985)
#Once the packages have been recalled, all their functions can be used. We use a population present in the package sampling
#This population provides information about the Swiss municipalities in 2003.
#This is a data frame with 2896 observations on 22 variables.
#To load the data set and obtain information on its variables, the following instructions must be executed.
 
data(swissmunicipalities)
?swissmunicipalities
dim(swissmunicipalities)
head(swissmunicipalities)

#where the data function allows to load the dataset in the R environment, 
#the question mark allows to receive information on the dataset and the head function
#allows to visualize the first 6 rows of the dataset.

#The REG variable (Swiss region) is made up of seven categories
#and it is used as a stratification variable. With the table function 
#the frequency distribution of the strata is obtained:

table(swissmunicipalities$REG)

#
#In order to realize a random stratified sampling in R
#it is necessary to order the frame according to the layers. 
#This operation can be performed through the order function:

data=swissmunicipalities
data=data[order(data$REG),]

# Once the data set for the variable indicating the 
# strata has been ordered, it is possible to extract the sample according to the different types of allocation

# n=210  (7%)

# n_1=n_2=n_3=...30
Nh<-table(data$REG)
N<-sum(Nh)
nh<-rep(30,7)
n<-210
# With the previous operations we define the size of the population (N),
# that of the sample (n), the number of each strata (Nh) and the sample size in each
# strata according to the type of allocation chosen. 
# In this example, we analyze the Airbat variable that represents the surface constructed in each municipality

# With the following instructions we calculate the semi-amplitude of the confidence interval
# The tapply function  divides the values of a quantitative variable for the levels of a qualitative variable and, 
# for each level, applies a particular function to the data

S2<-tapply(data$Airbat,data$REG,var)
varU<-sum((Nh/N)^2*((Nh-nh)/(Nh*nh))*S2)
deltaU<-1.96*sqrt(varU)*2
deltaU

# With the same functions previously analyzed, we can obtain the sample size in each strata with the proportional and 
# Neyman allocation

# Proportional
nh<-round(n*(Nh/N),0)
varP<-sum((Nh/N)^2*((Nh-nh)/(Nh*nh))*S2)
deltaP<-1.96*sqrt(varP)*2
deltaP

#Neyman
S<-tapply(data$Airbat,data$REG,sd)
nh<-round(n*Nh*S/(sum(Nh*S)))
varN<-sum((Nh/N)^2*((Nh-nh)/(Nh*nh))*S2)
deltaN<-1.96*sqrt(varN)*2
deltaN


#To compare the efficiency of the estimator obtained with the stratified sampling with respect to simple random sampling,
#the level of precision of the estimator obtained with simple random sampling is also computed
#srs
varSRS<-((N-n)/(N))*var(data$Airbat)/n
deltaSRS<-1.96*sqrt(varSRS)*2
deltaSRS

# It can be seen that stratified sampling allows to obtain more efficient estimators than simple random sampling

# The strata function allows to extract a stratified sample inserting as parameters the name of the dataset (data), 
# the name of the variable that identifies the layer (stratanames = c ("REG")), the sample size in each layer 
# (size = c ( nh)) and the extraction method in each layer (method = "srswor" - simple random sample). 
# With the getdata function the sample values 
# extracted from the population are obtained. The strata function also calculates the odds of inclusion ($ Prob).

st=strata(data,stratanames=c("REG"),size=c(nh), method="srswor")  
getdata(data, st)
table(st$REG) 
#Inclusion probabilities
pik<-nh/Nh
st$Prob
pik


#Three new variables are added to the extracted sample: the sample weight (pw), 
#the strata and the size of the population in each strata (fpc).
#At this point it is necessary to load the package survey into the R environment in order to be able to make the estimates.

library(survey)
sample.st<-cbind(data[st$ID_unit,],(1/st$Prob),st$Stratum,rep(Nh,nh))
names(sample.st)[23]<-"pw"
names(sample.st)[24]<-"strata"
names(sample.st)[25]<-"fpc"

# 
# The svydesign function allows to identify the type of sampling plan used: id = ~ 1 allows
# to declare that single municipalities have been sampled; fpc = ~ fpc indicates that the fpc variable contains
# the population size for each strata; strata = ~ layer defines the layers from where the sample units have been extracted;
# data = sample.st defines the data set that contains the data.


st_design<-svydesign(id=~1,strata=~strata,fpc=~fpc,data=sample.st)
st_design

#To calculate the average value and the total and the relative sample error (standard error) for any quantitative variable.

media<-svymean(~ Airbat,st_design,deff = TRUE)
media
totale<-svytotal(~ Airbat,st_design,deff = TRUE)
totale
confint(media,level=0.95)

#mean with con srs
meansrs=mean(sample.st$Airbat)
varsrs=sqrt(((N-n)/(N))*(var(sample.st$Airbat)/n))

meanpop=mean(data$Airbat)   # population mean



# what about the regression and the weighted regression? 
reg<-lm(data$Airbat~data$POPTOT)
summary(reg)


reg.srs<-lm(sample.st$Airbat~sample.st$POPTOT)
summary(reg.srs)
reg.st<-lm(sample.st$Airbat~sample.st$POPTOT,weights=sample.st$pw)
summary(reg.st)

# We can notice that using statified sampling the estimated values is closer the true vaue in the populaton and the
# se is smaller.


#######################################################################
#######################################################################
#######################################################################

# Assignment 1
# Given the data:
province <-
  read.table( text =
                "id str clu wt ue91 lab91 fpc
              1 1 1 1.75 4123 33786 7
              2 1 2 1.75 666 6016 7
              3 1 4 1.75 760 5919 7
              4 1 6 1.75 457 3022 7
              5 2 21 6.25 61 573 25
              6 2 25 6.25 262 1737 25
              7 2 26 6.25 331 2543 25
              8 2 27 6.25 98 545 25" ,
              header = TRUE
  )
# NOTE: In this data set, the fpc changes with the strata. 
# 1. Construct a survey.design object called province.design specifying a stratified random sampling design
# 2. View the weighted total population of this survey design, by referring to the wt column from the original 
# 3. Count the number of unweighted observations where the variable ue91 is not missing
# 4. Compute the mean and standard error of the ue91 variable
# 5. Compute the weighted total and standard error of the ue91 variable


# Assignment 2 
#The MSU284 Population (Sarndal et al., 2003) describes the 284 municipalities
# of Sweden. It is included in package sampling
# Obtain the mean of the population in 1985 under a simple random sampling and a stratified random sampling with the best allocation and compare it. 